import bids2openminds.converter
import os
from openminds import Collection
import pytest
from tempfile import mkdtemp
import pathlib
import shutil
import platform


dataset_labels = ["eeg_rest_fmri", "ds005"]


@pytest.fixture(scope="session")
def load_collections():
    collections = {}
    tempdir = mkdtemp()
    for dataset_label in dataset_labels:
        test_standard_name = "bids_examples_" + dataset_label + ".jsonld"
        test_standard_path = os.path.join("test", test_standard_name)

        test_dataset = os.path.join("bids-examples", dataset_label)
        prefix = pathlib.Path(test_dataset).absolute().as_uri()+"/"

        with open(test_standard_path, "r") as file:
            template_data = file.read()
            actual_data = template_data.replace(
                "PREFIX", prefix)

        with open(tempdir+test_standard_name, "w") as file:
            file.write(actual_data)

        reference_collection = Collection()
        reference_collection.load(tempdir+test_standard_name)

        bids2openminds.converter.convert(test_dataset, save_output=True)
        generated_collection = Collection()
        generated_collection.load(os.path.join(
            test_dataset, "openminds.jsonld"))

        collections[dataset_label] = (
            reference_collection, generated_collection)

    shutil.rmtree(tempdir)
    return collections


def detect_type(collection, type):
    type_IRI = "https://openminds.ebrains.eu/core/" + type
    items_list = []
    for item in collection:
        if item.type_ == type_IRI:
            items_list.append(item)
    return items_list if items_list else []


def find_lookup_label(lookup_label, items):
    for item in items:
        if item.lookup_label == lookup_label:
            return item
    return None


def find_name(name, items):
    for item in items:
        if item.name == name:
            return item
    return None


def find_person(given_name, family_name, persons):
    for person in persons:
        if person.given_name == given_name and person.family_name == family_name:
            return person
    return None


@pytest.mark.parametrize("dataset_label", dataset_labels)
def test_subject(load_collections, dataset_label):
    reference_collection, generated_collection = load_collections[dataset_label]
    generated_subjects = detect_type(generated_collection, "Subject")
    reference_subjects = detect_type(reference_collection, "Subject")

    assert len(generated_subjects) == len(reference_subjects)

    for reference_subject in reference_subjects:
        generated_subject = find_lookup_label(
            reference_subject.lookup_label, generated_subjects)
        assert generated_subject is not None


@pytest.mark.parametrize("dataset_label", dataset_labels)
def test_subject_state(load_collections, dataset_label):
    reference_collection, generated_collection = load_collections[dataset_label]
    generated_subjects_states = detect_type(
        generated_collection, "SubjectState")
    reference_subjects_states = detect_type(
        reference_collection, "SubjectState")

    assert len(generated_subjects_states) == len(reference_subjects_states)

    for reference_subject_state in reference_subjects_states:
        generated_subject_state = find_lookup_label(
            reference_subject_state.lookup_label, generated_subjects_states)
        assert generated_subject_state is not None


@pytest.mark.parametrize("dataset_label", dataset_labels)
def test_person(load_collections, dataset_label):
    reference_collection, generated_collection = load_collections[dataset_label]
    generated_persons = detect_type(generated_collection, "Person")
    reference_persons = detect_type(reference_collection, "Person")

    assert len(generated_persons) == len(reference_persons)

    for reference_person in reference_persons:
        generated_person = find_person(
            reference_person.given_name, reference_person.family_name, generated_persons)
        assert generated_person is not None


@pytest.mark.parametrize("dataset_label", dataset_labels)
def test_dataset_version(load_collections, dataset_label):
    reference_collection, generated_collection = load_collections[dataset_label]
    generated_dataset_version = detect_type(
        generated_collection, "DatasetVersion")[0]
    reference_dataset_version = detect_type(
        reference_collection, "DatasetVersion")[0]

    assert len(generated_dataset_version.experimental_approaches) == len(
        reference_dataset_version.experimental_approaches)
    assert generated_dataset_version.short_name == reference_dataset_version.short_name
    assert len(generated_dataset_version.techniques or []) == len(
        reference_dataset_version.techniques or [])


@pytest.mark.parametrize("dataset_label", dataset_labels)
def test_file(load_collections, dataset_label):
    reference_collection, generated_collection = load_collections[dataset_label]
    generated_files = detect_type(generated_collection, "File")
    reference_files = detect_type(reference_collection, "File")

    assert len(generated_files) == len(reference_files)
    for refrence_file in reference_files:
        generated_file = find_name(refrence_file.name, generated_files)
        assert generated_file is not None
        assert generated_file.iri.value == refrence_file.iri.value
        if generated_file.format is None:
            assert refrence_file.format is None
        else:
            assert generated_file.format.id == refrence_file.format.id
        if platform.system() != 'Windows':
            assert generated_file.hashes[0].digest == refrence_file.hashes[0].digest


@pytest.mark.parametrize("dataset_label", dataset_labels)
def test_file_bundle(load_collections, dataset_label):
    reference_collection, generated_collection = load_collections[dataset_label]
    generated_file_bundles = detect_type(generated_collection, "FileBundle")
    reference_file_bundles = detect_type(reference_collection, "FileBundle")

    assert len(generated_file_bundles) == len(reference_file_bundles)
    for refrence_file_bundle in reference_file_bundles:
        generated_file_bundle = find_name(
            refrence_file_bundle.name, generated_file_bundles)
        assert generated_file_bundle is not None
        if refrence_file_bundle.is_part_of.type_ == "https://openminds.ebrains.eu/core/FileBundle":
            assert generated_file_bundle.is_part_of.type_ == "https://openminds.ebrains.eu/core/FileBundle"
            assert refrence_file_bundle.is_part_of.name == generated_file_bundle.is_part_of.name



import hashlib
import json
import os
import re
import gzip
from warnings import warn

import pandas as pd

import openminds.v3.controlled_terms as controlled_terms
from openminds.v3.core import Hash, QuantitativeValue, ContentType
from openminds.v3.controlled_terms import UnitOfMeasurement


def read_json(file_path: str) -> dict:
    """
    Reads the content of a JSON file and returns it as a Python dictionary.

    Parameters:
    - file_path (str): The path to the JSON file.

    Returns:
    - dict: A Python dictionary containing the content of the JSON file.

    Example:
    >>> data = read_json('example.json')
    >>> print(data)
    {"Name": "The mother of all experiments" , "BIDSVersion": "1.6.0", "DatasetType": "raw" , "License": "CC0" , "Authors": ["Paul Broca" , "Carl Wernicke" ]}
    """
    try:
        # Open the JSON file
        with open(file_path, "r") as file:
            # Load the JSON content into a dictionary
            json_dic = json.load(file)
        return json_dic
    except FileNotFoundError:
        # Handle file not found error
        print(f"Error: File not found at {file_path}")
        return {}
    except json.JSONDecodeError:
        # Handle JSON decoding error
        print(f"Error: Unable to decode JSON content from {file_path}")
        return {}


def table_filter(dataframe: pd.DataFrame, filter_str: str, column: str = "suffix"):
    """
    Filters a Pandas DataFrame based on a specified condition.

    Parameters:
    - dataframe (pd.DataFrame): The DataFrame to be filtered.
    - filter_str (str): The value to filter the DataFrame on.
    - column (str, optional): The column name to apply the filter on. Default is "suffix".

    Returns:
    - pd.DataFrame: A filtered DataFrame containing only the rows that satisfy the condition.
    """
    try:
        # Apply the filter condition on the specified column
        filtered_dataframe = dataframe[dataframe[column] == filter_str]
        return filtered_dataframe
    except KeyError:
        # Handle the case where the specified column is not present in the DataFrame
        KeyError(f"Error: Column '{column}' not found in the DataFrame.")


def pd_table_value(data_frame, column_name, not_list: bool = True):
    try:
        if column_name in data_frame.columns:
            value = data_frame[column_name].to_list()
            if not_list:
                return value[0]
            else:
                return value
        else:
            return None
    except IndexError:
        warn(f"The data frame doesn't contain {column_name}")
        return None


def file_hash(file_path: str, algorithm: str = "MD5"):
    """
    Compute the hash digest of a file using the specified hashing algorithm an returns an openMINDs object.

    Parameters:
    - file_path (str): The path to the file for which you want to compute the hash.
    - algorithm (str, optional): The hashing algorithm to use. Default is "MD5".

    Returns:
    - Hash: An openMINDS object representing the computed hash, containing the algorithm and digest.
    """
    # Open the file in binary mode
    with open(file_path, "rb") as file:
        # Read the content of the file
        file_content = file.read()

        # Create a new hash object using the specified algorithm
        hash_object = hashlib.new(algorithm)

        # Update the hash object with the content of the file
        hash_object.update(file_content)

        # Calculate the hexadecimal digest of the hash
        hash_value = hash_object.hexdigest()

    # Create a openMINDS Hash object with the algorithm and digest
    openminds_hash = Hash(algorithm=algorithm, digest=hash_value)

    return openminds_hash


def file_storage_size(file_path: str):
    file_stats = os.stat(file_path)
    file_size = QuantitativeValue(
        value=file_stats.st_size, unit=UnitOfMeasurement.by_name("byte"))
    return file_size, file_stats.st_size


def detect_nifti_version(file_name, extension, file_size):

    nii1_sizeof_hdr = 348
    nii2_sizeof_hdr = 540

    if extension == ".nii":

        with open(file_name, 'rb') as fp:
            byte_data = fp.read(4)

        sizeof_hdr = int.from_bytes(byte_data, byteorder='little')

        if sizeof_hdr == 0:
            return None

        if sizeof_hdr == nii1_sizeof_hdr:
            return ContentType.by_name("application/vnd.nifti.1")

        elif sizeof_hdr == nii2_sizeof_hdr:
            return ContentType.by_name("application/vnd.nifti.2")

        else:  # big endian
            sizeof_hdr = int.from_bytes(byte_data, byteorder='big')

            if sizeof_hdr == nii1_sizeof_hdr:
                return ContentType.by_name("application/vnd.nifti.1")

            elif sizeof_hdr == nii2_sizeof_hdr:
                return ContentType.by_name("application/vnd.nifti.2")

    if extension == ".nii.gz":
        try:
            with gzip.open(file_name, 'rb') as fp:
                byte_data = fp.read(4)
        except gzip.BadGzipFile:
            return None

        sizeof_hdr = int.from_bytes(byte_data, byteorder='little')

        if sizeof_hdr == 0:
            return None

        if sizeof_hdr == nii1_sizeof_hdr:
            return ContentType.by_name("application/vnd.nifti.1")

        elif sizeof_hdr == nii2_sizeof_hdr:
            return ContentType.by_name("application/vnd.nifti.2")

        else:  # big endian
            sizeof_hdr = int.from_bytes(byte_data, byteorder='big')

            if sizeof_hdr == nii1_sizeof_hdr:
                return ContentType.by_name("application/vnd.nifti.1")

            elif sizeof_hdr == nii2_sizeof_hdr:
                return ContentType.by_name("application/vnd.nifti.2")

    return None



import pytest
from bids import BIDSLayout
from openminds import Collection
import os
from bids2openminds.main import create_behavioral_protocol

(datasets, tasks) = ("ds002", ("deterministicclassification",
                               "mixedeventrelatedprobe", "probabilisticclassification"))


@pytest.fixture
def creating_behavioral_protocols():
    c = Collection()
    test_dir = os.path.join("bids-examples", datasets)
    bids_layout = BIDSLayout(test_dir)
    _, behavioral_protocols_dict = create_behavioral_protocol(
        bids_layout, c)
    return behavioral_protocols_dict


def test_behavioral_protocols(creating_behavioral_protocols):
    assert len(tasks) == len(creating_behavioral_protocols)
    for item in creating_behavioral_protocols:
        assert item in tasks
        assert creating_behavioral_protocols[item].internal_identifier == item



import pandas as pd
import pytest
from bids2openminds.main import create_openminds_age

example_ages = [("89+", "QuantitativeValueRange"),
                (45, "QuantitativeValue"), ("XX", None)]


@ pytest.mark.parametrize("age,age_type", example_ages)
def test_subject_age(age, age_type):
    data_subject_table = pd.DataFrame(data={'age': [age]})
    openminds_age = create_openminds_age(data_subject_table)
    if age_type == "QuantitativeValueRange":
        assert openminds_age.type_ == 'https://openminds.ebrains.eu/core/QuantitativeValueRange'
        assert openminds_age.max_value is None
        assert openminds_age.min_value == 89
    if age_type == "QuantitativeValue":
        assert openminds_age.type_ == 'https://openminds.ebrains.eu/core/QuantitativeValue'
        assert openminds_age.value == age
    if age_type == None:
        assert openminds_age is None



import re
import os
import pathlib
from warnings import warn

import pandas as pd
from nameparser import HumanName

import openminds.v3.core as omcore
import openminds.v3.controlled_terms as controlled_terms
from openminds import IRI

from .utility import table_filter, pd_table_value, file_hash, file_storage_size, detect_nifti_version
from . import mapping


def create_openminds_person(full_name):
    # Regex for detecting any unwanted characters.
    name_regex = re.compile(
        r"^[\w'\-, .][^0-9_!¡?÷?¿/\\+=@#$%^&*(){}|~<>;:[\]]{1,}$")
    alternate_names = []
    person = HumanName(full_name)
    given_name = person.first
    family_name = person.last

    # Handle the situation in which there is no given name or the given name consists of unwanted characters.
    if not (given_name and name_regex.match(given_name)):
        if not (len(family_name) > 0 and len(given_name) == 1):
            return None

    # Handle the situation in which the family name consists of unwanted characters.
    if not (name_regex.match(family_name)):
        family_name = None

    if person.middle:
        given_name = f"{given_name} {person.middle}"

    if person.nickname:
        alternate_names.append(person.nickname)

    if not alternate_names:
        alternate_names = None

    openminds_person = omcore.Person(
        alternate_names=alternate_names, given_name=given_name, family_name=family_name)

    return openminds_person


def create_persons(dataset_description, collection):

    if "Authors" in dataset_description:
        person_list = dataset_description["Authors"]
    else:
        return None

    if not (isinstance(person_list, list)):
        # handel's only one name
        if isinstance(person_list, str):
            openminds_person = create_openminds_person(person_list)
            if openminds_person is not None:
                collection.add(openminds_person)
            return openminds_person
        else:
            return None

    openminds_list = []
    for person in person_list:
        openminds_person = create_openminds_person(person)
        if openminds_person is not None:
            openminds_list.append(openminds_person)
            collection.add(openminds_person)

    return openminds_list


def create_behavioral_protocol(layout, collection):
    behavioral_protocols_dict = {}
    behavioral_protocols = []
    tasks = layout.get_tasks()

    if not tasks:
        return None, None

    for task in tasks:

        behavioral_protocol = omcore.BehavioralProtocol(name=task,
                                                        internal_identifier=task,
                                                        description="To be defined")
        behavioral_protocols.append(behavioral_protocol)
        behavioral_protocols_dict[task] = behavioral_protocol
        collection.add(behavioral_protocol)

    return behavioral_protocols, behavioral_protocols_dict


def techniques_openminds(suffix):
    possible_types = ["Technique", "AnalysisTechnique", "StimulationApproach",
                      "StimulationTechnique"]

    if suffix in mapping.MAP_2_TECHNIQUES:
        items_openminds = mapping.MAP_2_TECHNIQUES[suffix]
    else:
        return []

    if items_openminds is None:
        return []

    openminds_techniques_list = []
    for item_openminds in items_openminds:
        for possible_type in possible_types:
            openminds_type = getattr(controlled_terms, possible_type)
            openminds_obj = None
            try:
                openminds_obj = openminds_type.by_name(item_openminds)
                break
            except KeyError:
                pass
        if openminds_obj:
            openminds_techniques_list.append(openminds_obj)
    return openminds_techniques_list


def create_techniques(layout_df):
    suffixs = layout_df["suffix"].unique().tolist()
    techniques = []
    not_techniques_index = ["description", "participants", "events"]
    for suffix in suffixs:
        # excluding the None and non thechnique indexes
        if not (pd.isnull(suffix) or (suffix in not_techniques_index)):
            openminds_techniques_cache = techniques_openminds(suffix)
            techniques.extend(openminds_techniques_cache)

    techniques_set = set(techniques)
    techniques_list = list(techniques_set)
    return techniques_list or None


def approaches_openminds(datatype):

    if datatype in mapping.MAP_2_EXPERIMENTAL_APPROACHES:
        items_openminds = mapping.MAP_2_EXPERIMENTAL_APPROACHES[datatype]

    approches_list = []

    for item in items_openminds:
        approches_list.append(
            controlled_terms.ExperimentalApproach.by_name(item))

    return approches_list


def create_approaches(layout_df):
    datatypes = layout_df["datatype"].unique().tolist()
    approaches = set([])
    for datatype in datatypes:
        if not (pd.isnull(datatype)):
            approaches.update(approaches_openminds(datatype))

    return list(approaches) or None


def create_openminds_age(data_subject):

    try:
        age = pd_table_value(data_subject, "age")
    except:
        return None

    if age is None or pd.isna(age):
        return None
    elif isinstance(age, float) or isinstance(age, int) or age.isnumeric():
        return omcore.QuantitativeValue(
            value=age,
            unit=controlled_terms.UnitOfMeasurement.year
        )
    elif age == "89+":
        return omcore.QuantitativeValueRange(
            max_value=None,
            min_value=89,
            min_value_unit=controlled_terms.UnitOfMeasurement.year
        )
    else:
        return None


def create_dataset_version(bids_layout, dataset_description, layout_df, studied_specimens, file_repository, behavioral_protocols, collection):

    # Fetch the dataset type from dataset description file

    # dataset_type=bids2openminds_instance(dataset_description.get("DatasetType",None))

    # Fetch the digitalIdentifier from dataset description file

    if "DatasetDOI" in dataset_description:
        digital_identifier = omcore.DOI(
            identifier=dataset_description["DatasetDOI"])
    else:
        digital_identifier = None

    authors = create_persons(dataset_description, collection)

    if "Acknowledgements" in dataset_description:
        other_contribution = dataset_description["Acknowledgements"]
    else:
        other_contribution = None

    if "HowToAcknowledge" in dataset_description:
        how_to_cite = dataset_description["HowToAcknowledge"]
    else:
        how_to_cite = None

    if ("DatasetType" in dataset_description) and (dataset_description == "derivative"):
        dataset_type = controlled_terms.SemanticDataType.derived_data
    else:
        dataset_type = controlled_terms.SemanticDataType.raw_data

    # TODO funding
    # if "Funding" in dataset_description:
    #   funding=funding_openMINDS(dataset_description["Funding"])
    # else:
    #   funding=None

    # if "EthicsApprovals" in dataset_description:
    #   #to be completed ethics_assessment
    #   ethics_assessment=controlledTerms.EthicsAssessment.by_name("EU compliant")
    # else:
    #   ethics_assessment=None

    # creating a list containing all the Modalities used in this dataset

    techniques = create_techniques(layout_df)

    experimental_approaches = create_approaches(layout_df)

    dataset_version = omcore.DatasetVersion(
        digital_identifier=digital_identifier,
        experimental_approaches=experimental_approaches,
        short_name=dataset_description["Name"],
        full_name=dataset_description["Name"],
        studied_specimens=studied_specimens,
        authors=authors,
        techniques=techniques,
        how_to_cite=how_to_cite,
        repository=file_repository,
        behavioral_protocols=behavioral_protocols,
        data_types=dataset_type
        # other_contributions=other_contribution  # needs to be a Contribution object
        # version_identifier
    )

    collection.add(dataset_version)

    return dataset_version


def create_dataset(dataset_description, dataset_version, collection):

    dataset = omcore.Dataset(
        digital_identifier=dataset_version.digital_identifier,
        authors=dataset_version.authors,
        full_name=dataset_version.full_name,
        short_name=dataset_version.short_name,
        has_versions=dataset_version
    )

    collection.add(dataset)

    return dataset


def spices_openminds(data_subject: pd.DataFrame):
    bids_species = pd_table_value(data_subject, "species")
    if bids_species is None:
        # In BIDS the default species is homo sapiens.
        return controlled_terms.Species.homo_sapiens
    if bids_species in mapping.MAP_2_SPECIES:
        openminds_species = mapping.MAP_2_SPECIES[bids_species]
        return controlled_terms.Species.by_name(openminds_species[0])
    else:
        try:
            openminds_species = controlled_terms.Species.by_name(bids_species)
            warn(
                f"You have specified {bids_species} as species, we have autodetected {openminds_species.name}, please verify it.")
            return openminds_species
        except KeyError:
            warn(
                f"You have specified {bids_species} we currently don't support this species.")
        return None


def handedness_openminds(data_subject: pd.DataFrame):
    bids_handedness = pd_table_value(data_subject, "handedness")
    if bids_handedness is None:
        return None
    if bids_handedness in mapping.MAP_2_HANDEDNESS:
        openminds_handedness = mapping.MAP_2_HANDEDNESS[bids_handedness]
        return controlled_terms.Handedness.by_name(openminds_handedness[0])
    else:
        warn(
            f"You have specified {bids_handedness} which is not a allowed value for handedness defined by BIDS standard.")
        return None


def sex_openminds(data_subject: pd.DataFrame):
    bids_sex = pd_table_value(data_subject, "sex")
    if bids_sex is None:
        return None
    if bids_sex in mapping.MAP_2_BIOLOGICALSEX:
        bids_sex = mapping.MAP_2_BIOLOGICALSEX[bids_sex]
        return controlled_terms.BiologicalSex.by_name(bids_sex[0])
    else:
        warn(
            f"You have specified {bids_sex} which is not a allowed value for handedness defined by BIDS standard.")
        return None


def create_subjects(subject_id, layout_df, layout, collection):

    sessions = layout.get_sessions()
    subjects_dict = {}
    subjects_list = []
    subject_state_dict = {}

    # Find the participants files in the files table
    participants_paths = table_filter(layout_df, "participants")
    if participants_paths.empty:
        # creating emphty subjects just based on file structure
        for subject in subject_id:
            subject_name = f"sub-{subject}"
            state_cache_dict = {}
            state_cache = []
            # dealing with condition that have no seasion
            if not sessions:
                state = omcore.SubjectState(
                    internal_identifier=f"Studied state {subject_name}".strip(
                    ),
                    lookup_label=f"Studied state {subject_name}".strip()
                )
                collection.add(state)
                state_cache_dict[""] = state
                state_cache.append(state)
            else:
                # create a subject state for each state
                for session in sessions:
                    if not (table_filter(table_filter(layout_df, session, "session"), subject, "subject").empty):
                        state = omcore.SubjectState(
                            internal_identifier=f"Studied state {subject_name} {session}".strip(
                            ),
                            lookup_label=f"Studied state {subject_name} {session}".strip(
                            )
                        )
                        collection.add(state)
                        state_cache_dict[f"{session}"] = state
                        state_cache.append(state)
            subject_state_dict[f"{subject}"] = state_cache_dict
            subject_cache = omcore.Subject(
                lookup_label=f"{subject_name}",
                internal_identifier=f"{subject_name}",
                studied_states=state_cache
            )
            subjects_dict[f"{subject}"] = subject_cache
            subjects_list.append(subject_cache)
            collection.add(subject_cache)

        return subjects_dict, subject_state_dict, subjects_list

    # Select the tsv file of the table
    participants_path_tsv = pd_table_value(table_filter(
        participants_paths, ".tsv", "extension"), "path")

    participants_table = pd.read_csv(participants_path_tsv, sep="\t", header=0)
    for subject in subject_id:
        subject_name = f"sub-{subject}"
        data_subject = table_filter(
            participants_table, subject_name, "participant_id")
        state_cache_dict = {}
        state_cache = []
        if not sessions:
            state = omcore.SubjectState(
                age=create_openminds_age(data_subject),
                handedness=handedness_openminds(data_subject),
                internal_identifier=f"Studied state {subject_name}".strip(),
                lookup_label=f"Studied state {subject_name}".strip()
            )
            collection.add(state)
            state_cache_dict[""] = state
            state_cache.append(state)
        else:
            for session in sessions:
                if not (table_filter(table_filter(layout_df, session, "session"), subject, "subject").empty):
                    state = omcore.SubjectState(
                        age=create_openminds_age(data_subject),
                        handedness=handedness_openminds(data_subject),
                        internal_identifier=f"Studied state {subject_name} {session}".strip(
                        ),
                        lookup_label=f"Studied state {subject_name} {session}".strip(
                        )
                    )
                    collection.add(state)
                    state_cache_dict[f"{session}"] = state
                    state_cache.append(state)
            subject_state_dict[f"{subject}"] = state_cache_dict
        subject_cache = omcore.Subject(
            biological_sex=sex_openminds(data_subject),
            lookup_label=f"{subject_name}",
            internal_identifier=f"{subject_name}",
            # TODO species should default to homo sapiens
            species=spices_openminds(data_subject),
            studied_states=state_cache
        )
        subjects_dict[f"{subject}"] = subject_cache
        subjects_list.append(subject_cache)
        collection.add(subject_cache)

    return subjects_dict, subject_state_dict, subjects_list


def create_file_bundle(BIDS_path, path, collection, parent_file_bundle=None, is_file_repository=False):

    if is_file_repository:
        openminds_file_bundle = omcore.FileRepository(format=omcore.ContentType.by_name("application/vnd.bids"),
                                                      iri=IRI(pathlib.Path(BIDS_path).absolute().as_uri()))
    else:
        relative_path = os.path.relpath(path, BIDS_path)
        name = str(relative_path).replace("\\", "/")
        if name[0] == "_":
            name = name[1:]
        openminds_file_bundle = omcore.FileBundle(content_description=f"File bundle created for {relative_path}",
                                                  name=name,
                                                  is_part_of=parent_file_bundle)

    files = {}
    files_size = 0
    all = os.listdir(path)

    for item in all:

        item_path = str(pathlib.PurePath(path, item))

        if os.path.isfile(item_path) and os.path.basename(item_path) != "openminds.jsonld":

            if is_file_repository:
                files[item_path] = None
            else:
                files[item_path] = [openminds_file_bundle]

            files_size += os.stat(item_path).st_size

        if os.path.isdir(item_path) and os.path.basename(item_path) != "openminds":

            child_files, child_filesizes, _ = create_file_bundle(
                BIDS_path, item_path, collection, parent_file_bundle=openminds_file_bundle, is_file_repository=False)

            for child_file_path in child_files.keys():
                if child_file_path not in files:
                    files[child_file_path] = []

                files[child_file_path].extend(child_files[child_file_path])

            files_size += child_filesizes

    openminds_file_bundle.storage_size = omcore.QuantitativeValue(value=files_size,
                                                                  unit=controlled_terms.UnitOfMeasurement.by_name(
                                                                      "byte")
                                                                  )
    collection.add(openminds_file_bundle)

    if is_file_repository:
        openminds_file_repository = openminds_file_bundle
    else:
        openminds_file_repository = None

    return files, files_size, openminds_file_repository


def create_file(layout_df, BIDS_path, collection):

    BIDS_path_absolute = pathlib.Path(BIDS_path).absolute()

    file2file_bundle_dic, _, file_repository = create_file_bundle(
        BIDS_path_absolute, BIDS_path_absolute, collection, is_file_repository=True)

    files_list = []
    for index, file in layout_df.iterrows():
        file_format = None
        content_description = None
        data_types = None
        extension = file["extension"]
        path = file["path"]
        iri = IRI(pathlib.Path(path).absolute().as_uri())
        name = os.path.basename(path)
        hashes = file_hash(path)
        storage_size_obj, file_size = file_storage_size(path)
        if pd.isna(file["subject"]):
            if file["suffix"] == "participants":
                if extension == ".json":
                    content_description = f"A JSON metadata file of participants TSV."
                    data_types = controlled_terms.DataType.by_name(
                        "associative array")
                    file_format = omcore.ContentType.by_name(
                        "application/json")
                elif extension == [".tsv"]:
                    content_description = f"A metadata table for participants."
                    data_types = controlled_terms.DataType.by_name("table")
                    file_format = omcore.ContentType.by_name(
                        "text/tab-separated-values")
        else:
            if extension == ".json":
                content_description = f"A JSON metadata file for {file['suffix']} of subject {file['subject']}"
                data_types = controlled_terms.DataType.by_name(
                    "associative array")
                file_format = omcore.ContentType.by_name("application/json")
            elif extension in [".nii", ".nii.gz"]:
                content_description = f"Data file for {file['suffix']} of subject {file['subject']}"
                data_types = controlled_terms.DataType.by_name("voxel data")
                file_format = detect_nifti_version(path, extension, file_size)
            elif extension == [".tsv"]:
                if file["suffix"] == "events":
                    content_description = f"Event file for {file['suffix']} of subject {file['subject']}"
                    data_types = controlled_terms.DataType.by_name(
                        "event sequence")
                    file_format = omcore.ContentType.by_name(
                        "text/tab-separated-values")

        file = omcore.File(
            iri=iri,
            content_description=content_description,
            data_types=data_types,
            file_repository=file_repository,
            format=file_format,
            hashes=hashes,
            is_part_of=file2file_bundle_dic[str(
                pathlib.Path(path))],
            name=name,
            # special_usage_role
            storage_size=storage_size_obj,
        )
        collection.add(file)
        files_list.append(file)

    return files_list, file_repository



import os


def create_report(dataset, dataset_version, collection, dataset_description, input_path, output_path):
    subject_number = 0
    subject_state_numbers = []
    file_bundle_number = 0
    files_number = 0
    behavioral_protocols_numbers = 0

    for item in collection:
        if item.type_.endswith("Subject"):

            subject_number += 1
            subject_state_numbers.append(len(item.studied_states))

        if item.type_.endswith("File"):

            files_number += 1

        if item.type_.endswith("FileBundle"):

            file_bundle_number += 1

        if item.type_.endswith("BehavioralProtocol"):

            behavioral_protocols_numbers += 1

    experimental_approaches_list = ""
    if dataset_version.experimental_approaches is not None:
        for approache in dataset_version.experimental_approaches:
            experimental_approaches_list += f"{approache.name}\n"

    data_types_list = ""
    if dataset_version.data_types is not None:
        if isinstance(dataset_version.data_types, list):
            for data_type in dataset_version.data_types:
                data_types_list += f"{data_type.name}\n"
        else:
            data_types_list = f"{dataset_version.data_types.name}\n"

    author_list = ""
    i = 1
    if dataset_version.authors is not None:
        for author in dataset_version.authors:
            if author.family_name is not None:
                author_list += f"  {i}. {author.family_name}, {author.given_name}\n"
                i += 1
            else:
                author_list += f"  {i}. ___, {author.given_name}\n"
                i += 1

    min_subject_state_numbers = min(subject_state_numbers)
    max_subject_state_numbers = max(subject_state_numbers)
    if min_subject_state_numbers == max_subject_state_numbers:
        text_subject_state_numbers = str(min_subject_state_numbers)
    else:
        text_subject_state_numbers = f"min={min_subject_state_numbers}, max={max_subject_state_numbers}"

    report = f"""
Conversion Report
=================  
Conversion was successful, the openMINDS file is in {output_path}

Dataset title : {dataset.full_name}


Experimental approaches detected:
------------------------------------------ 
{experimental_approaches_list}

Detected data types:
------------------------------------------
{data_types_list}

The following elements were converted:  
------------------------------------------   
+ number of authors : {len(dataset_version.authors or [])}
+ number of converted subjects: {subject_number}  
+ number of states per subject: {text_subject_state_numbers}
+ number of files: {files_number} 
+ number of file bundles: {file_bundle_number}
+ number of techniques: {len(dataset_version.techniques or [])}
+ number of behavioral protocols: {behavioral_protocols_numbers}



**Important Notes**
------------------------------------------ 

Authors:
    The conversion of authors is not reliable due to missing source convention.
    The converter may fail in detecting family vs given name.
    The converter will fail in detecting organizations.
    The following persons (family name, given name) were converted, : 
{author_list}

Subject States:
    There are as many subject states as sessions for each subject.
    Please modify to your needs (divide into more or merge into fewer subject states).

Behavioral protocols:
    The conversion of behavioral protocols is incomplete.
    Only the task-label is extracted as name and internal identifier of a behavioral protocol.
    Please adjust to your needs.
 
"""
    if "GeneratedBy" in dataset_description:
        report = report+"+Dataset is derivative, derivative data are ignored for now\n"

    derivatives_path = os.path.join(
        input_path, "derivatives")
    if os.path.isdir(derivatives_path):
        report = report+"+ Dataset contains derivative, derivative data are ignored for now\n"

    return report



import os
import shutil
from openminds import Collection
from bids2openminds.converter import convert_click
from click.testing import CliRunner

(test_data_set, number_of_openminds_files) = ("ds003", 143)


def test_example_datasets_click():
    test_dir = os.path.join("bids-examples", test_data_set)
    runner = CliRunner()
    result = runner.invoke(convert_click, [test_dir])
    assert result.exit_code == 0
    c = Collection()
    c.load(os.path.join(test_dir, "openminds.jsonld"))


def test_example_datasets_click_seperate_files():
    test_dir = os.path.join("bids-examples", test_data_set)
    path_openminds = os.path.join(test_dir, "openminds")
    if os.path.isdir(path_openminds):
        shutil.rmtree(path_openminds)
    runner = CliRunner()
    result = runner.invoke(convert_click, ["--multiple-files", test_dir])
    assert result.exit_code == 0
    numer_of_files = len(os.listdir(path_openminds))
    assert numer_of_files == number_of_openminds_files


def test_example_datasets_click_output_location():
    test_dir = os.path.join("bids-examples", test_data_set)
    openminds_file = os.path.join("bids-examples", "test_openminds.jsonld")
    runner = CliRunner()
    result = runner.invoke(
        convert_click, ["-o", openminds_file, test_dir])
    assert result.exit_code == 0
    c = Collection()
    c.load(openminds_file)



import warnings
from bids import BIDSLayout, BIDSValidator
from openminds import Collection
import os
import click
from . import main
from . import utility
from . import report


def convert(input_path,  save_output=False, output_path=None, multiple_files=False, include_empty_properties=False, quiet=False):
    if not (os.path.isdir(input_path)):
        raise NotADirectoryError(
            f"The input directory is not valid, you have specified {input_path} which is not a directory."
        )
    # TODO use BIDSValidator to check if input directory is a valid BIDS directory
    # if not(BIDSValidator().is_bids(input_path)):
    #  raise NotADirectoryError(f"The input directory is not valid, you have specified {input_path} which is not a BIDS directory.")

    if quiet:
        warnings.filterwarnings('ignore')

    collection = Collection()
    bids_layout = BIDSLayout(input_path)

    layout_df = bids_layout.to_df()

    subjects_id = bids_layout.get_subjects()

    # imprting the dataset description file containing some of the
    dataset_description_path = utility.table_filter(layout_df, "description")

    dataset_description = utility.read_json(dataset_description_path.iat[0, 0])

    [subjects_dict, subject_state_dict, subjects_list] = main.create_subjects(
        subjects_id, layout_df, bids_layout, collection)

    behavioral_protocols, behavioral_protocols_dict = main.create_behavioral_protocol(
        bids_layout, collection)

    [files_list, file_repository] = main.create_file(
        layout_df, input_path, collection)

    dataset_version = main.create_dataset_version(
        bids_layout, dataset_description, layout_df, subjects_list, file_repository, behavioral_protocols, collection)

    dataset = main.create_dataset(
        dataset_description, dataset_version, collection)

    failures = collection.validate(ignore=["required", "value"])
    assert len(failures) == 0

    if save_output:
        if output_path is None:
            if multiple_files:
                output_path = os.path.join(input_path, "openminds")
            else:
                output_path = os.path.join(input_path, "openminds.jsonld")

        collection.save(output_path, individual_files=multiple_files,
                        include_empty_properties=include_empty_properties)

    if not quiet:
        print(report.create_report(dataset, dataset_version, collection,
                                   dataset_description, input_path, output_path))

    else:
        print("Conversion was successful")

    return collection


@click.command()
@click.argument("input-path", type=click.Path(file_okay=False, exists=True))
@click.option("-o", "--output-path", default=None, type=click.Path(file_okay=True, writable=True), help="The output path or filename for OpenMINDS file/files.")
@click.option("--single-file", "multiple_files", flag_value=False, default=False, help="Save the entire collection into a single file (default).")
@click.option("--multiple-files", "multiple_files", flag_value=True, help="Each node is saved into a separate file within the specified directory. 'output-path' if specified, must be a directory.")
@click.option("-e", "--include-empty-properties", is_flag=True, default=False, help="Whether to include empty properties in the final file.")
@click.option("-q", "--quiet", is_flag=True, default=False, help="Not generate the final report and no warning.")
def convert_click(input_path, output_path, multiple_files, include_empty_properties, quiet):
    convert(input_path, save_output=True, output_path=output_path,
            multiple_files=multiple_files, include_empty_properties=include_empty_properties, quiet=quiet)


if __name__ == "__main__":
    input_path = input("Enter the BIDS directory path: ")
    convert(input_path, save_output=True)



import pytest
import os
import pathlib
from openminds import Collection
from bids2openminds.main import create_file_bundle

(test_data_set, number_of_openminds_bundle) = (
    "ds007", 60)

# example_file_filebundle = ([list of the path to this file],[expected file bundle])
example_file_filebundle = ((["sub-04", "anat", "sub-04_inplaneT2.nii.gz"], ["sub-04/anat"]),
                           (["sub-03", "func", "sub-03_task-stopsignalwithmanualresponse_run-02_events.tsv"], ["sub-03/func"]))
# example_folder_filebundle= ([list of path to this folder],[expected folder name],[expected parent filebundle (None for highest level file bundles)])
example_folder_filebundle = ((["sub-04", "anat"], ["sub-04/anat"], ["sub-04"]),
                             (["sub-01"], ["sub-01"], [None]))


@pytest.fixture(scope="session")
def test_dir():
    test_dir = os.path.join("bids-examples", test_data_set)
    return test_dir


def path_name(path):
    name = str(path).replace("\\", "/")
    return name


def example_path(test_dir, path_list):
    path = test_dir
    for item in path_list:
        path = os.path.join(path, item)
    return path


@pytest.fixture(scope="session")
def generate_file_bundle_collection(test_dir):
    collection = Collection()
    file_bundles, _, file_repository = create_file_bundle(
        test_dir, test_dir, collection, is_file_repository=True)
    return file_bundles, collection, file_repository


def test_file_bundles_type(generate_file_bundle_collection):
    file_bundles, _, _ = generate_file_bundle_collection
    assert type(file_bundles) is dict


def test_number_file_bundle(generate_file_bundle_collection):
    _, collection, _ = generate_file_bundle_collection
    m = 0
    for item in collection:
        if item.type_ == "https://openminds.ebrains.eu/core/FileBundle":
            m += 1
    assert m == number_of_openminds_bundle


@pytest.mark.parametrize("path_list, bundle", example_file_filebundle)
def test_random_file(test_dir, path_list, bundle, generate_file_bundle_collection):

    file_bundles, _, _ = generate_file_bundle_collection

    example_file_path = example_path(test_dir, path_list)

    test_bundles = file_bundles[example_file_path]

    assert len(test_bundles) == len(bundle)

    for test_bundle in test_bundles:
        assert test_bundle.name in bundle


@pytest.mark.parametrize("path_list, bundle, parent_bundle", example_folder_filebundle)
def test_random_folder(test_dir, path_list, bundle, parent_bundle, generate_file_bundle_collection):

    _, collection, _ = generate_file_bundle_collection

    file_repository_iri = pathlib.Path(test_dir).absolute().as_uri()

    folder_name = path_name(example_path("", path_list))

    dataset_bundle = None

    for item in collection:
        if item.type_ == "https://openminds.ebrains.eu/core/FileBundle" and item.name == folder_name:
            # detects only one file bundle have this name
            assert dataset_bundle is None
            dataset_bundle = item
    # asserts at least one file bundle have this name
    assert dataset_bundle is not None

    assert dataset_bundle.name == bundle[0]

    if parent_bundle is not None:
        assert dataset_bundle.is_part_of.name == parent_bundle[0]
    else:
        assert dataset_bundle.is_part_of.type_ == "https://openminds.ebrains.eu/core/FileRepository"
        assert dataset_bundle.is_part_of.iri.value == file_repository_iri



import os
import pytest
from openminds import Collection
import bids2openminds.converter


# Dataset information in following order dataset_label, dataset_subject_number, dataset_subject_state_number, dataset_person_number, dataset_files_number, dataset_file_bundles_number, dataset_behavioral_protocol_number
example_dataset = [("ds003", 13, 13, 2, 58, 39, 1),
                   ("ds000247", 6, 10, 5, 202, 41, 2),
                   # The authors list in 'eeg_cbm' contains non person entities 2 is not correct name (issue raied #43)
                   ("eeg_cbm", 20, 20, 2, 104, 40, 1),
                   ("asl001", 1, 1, 2, 8, 3, 0),
                   # Number of files in 'eeg_rest_fmri' is not correct as it doesn't contain files in derivated (issue raied #42)
                   ("eeg_rest_fmri", 3, 3, 6, 45, 22, 1)]


@pytest.mark.parametrize("dataset_label, dataset_subject_number, dataset_subject_state_number, dataset_person_number, dataset_files_number, dataset_file_bundles_number, dataset_behavioral_protocol_number", example_dataset)
def test_example_datasets(dataset_label, dataset_subject_number, dataset_subject_state_number, dataset_person_number, dataset_files_number, dataset_file_bundles_number, dataset_behavioral_protocol_number):
    test_dir = os.path.join("bids-examples", dataset_label)
    bids2openminds.converter.convert(test_dir, save_output=True)
    c = Collection()
    c.load(os.path.join(test_dir, "openminds.jsonld"))

    subject_number = 0
    subject_state_number = 0
    person_number = 0
    files_number = 0
    file_bundles_number = 0
    behavioral_protocol_number = 0
    subject_state_number_besed_on_subject = 0

    for item in c:
        if item.type_ == "https://openminds.ebrains.eu/core/Subject":
            subject_number += 1
            subject_state_number_besed_on_subject += len(item.studied_states)
        if item.type_ == "https://openminds.ebrains.eu/core/SubjectState":
            subject_state_number += 1
        if item.type_ == "https://openminds.ebrains.eu/core/Person":
            person_number += 1
        if item.type_ == "https://openminds.ebrains.eu/core/File":
            files_number += 1
        if item.type_ == "https://openminds.ebrains.eu/core/FileBundle":
            file_bundles_number += 1
        if item.type_ == "https://openminds.ebrains.eu/core/BehavioralProtocol":
            behavioral_protocol_number += 1

    assert dataset_subject_number == subject_number
    assert dataset_subject_state_number == subject_state_number
    assert subject_state_number_besed_on_subject == subject_state_number, "There was a discrepancy between the total number of subject states and the subject states attached to subjects."
    assert dataset_person_number == person_number
    assert dataset_files_number == files_number
    assert dataset_file_bundles_number == file_bundles_number
    assert dataset_behavioral_protocol_number == behavioral_protocol_number



MAP_2_EXPERIMENTAL_APPROACHES = {
    "func": ["neuroimaging"],
    "dwi": [
        "neuroimaging",
        "neural connectivity",
        "anatomy"
    ],
    "fmap": ["neuroimaging"],
    "anat": [
        "neuroimaging",
        "anatomy"
    ],
    "perf": [
        "neuroimaging",
        "anatomy"
    ],
    "meg": ["neuroimaging"],
    "eeg": ["electrophysiology"],
    "ieeg": ["electrophysiology"],
    "beh": ["behavior"],
    "pet": [
        "neuroimaging",
        "radiology"
    ],
    "micr": [
        "microscopy",
        "anatomy",
        "histology"
    ],
    "nirs": ["neuroimaging"]
}

MAP_2_TECHNIQUES = {
    "angio": ["angiography"],
    "M0map": ["equilibrium magnetization mapping"],
    "FLASH": ["fast-low-angle-shot pulse sequence"], #TODO instance TBD
    "FLAIR": ["fluid attenuated inversion recovery pulse sequence"], #TODO instance TBD
    "UNIT1": None, #TODO instance TBD
    "inplaneT1": [
        "T1 pulse sequence", 
        "structural magnetic resonance imaging"
    ], #TODO sMRI
    "inplaneT2": [
        "T2 pulse sequence", 
        "structural magnetic resonance imaging"
    ], #TODO sMRI
    "R1map": None, #TODO instance TBD
    "T1map": None, #TODO instance TBD
    "MTVmap": [
        "quantitative magnetic resonance imaging",
        "macromolecular tissue volume image processing"
    ], #TODO other?
    "MTRmap": [
        "magnetization transfer imaging",
        "magnetization transfer ratio image processing",
        "magnetization transfer pulse sequence"
    ], #TODO instances
    "MTsat": [
        "magnetization transfer imaging",
        "magnetization transfer saturation image processing",
        "magnetization transfer pulse sequence"
    ], #TODO instances
    "MWFmap": [
        "myelin water imaging",
        "T2 pulse sequence",
        "myelin water fraction image processing"
    ], #TODO instances
    "S0map": None, #TODO instances
    "R2starmap": None, #TODO instance TBD
    "T2starmap": None, #TODO instance TBD
    "PDT2": None, #TODO instance TBD
    "PDw": None, #TODO instance TBD
    "PD": None, #TODO instance TBD
    "PDmap": None, #TODO instance TBD
    "Chimap": None, #TODO instance TBD
    "RB1map": None, #TODO instance TBD
    "TB1map": None, #TODO instance TBD
    "T1rho": None, #TODO instance TBD
    "T1w": None, #TODO instance TBD
    "T2w": None, #TODO instance TBD
    "T2star": None, #TODO instance TBD
    "T2starw": None, #TODO instance TBD
    "R2map": None, #TODO instance TBD
    "T2map": None, #TODO instance TBD
    "bold": None, #TODO instance TBD
    "cbv": None, #TODO instance TBD
    "phase": None, #TODO instance TBD
    "defacemask": None, #TODO instance TBD
    "epi": None, #TODO instance TBD
    "fieldmap": None, #TODO instance TBD
    "magnitude": None, #TODO instance TBD
    "magnitude1": None, #TODO instance TBD
    "magnitude2": None, #TODO instance TBD
    "phase1": None, #TODO instance TBD
    "phase2": None, #TODO instance TBD
    "phasediff": None, #TODO instance TBD
    "dwi": ["diffusion-weighted imaging"],
    "sbref": None, #TODO instance TBD
    "asl": None, #TODO instance TBD
    "m0scan": None, #TODO instance TBD
    "eeg": ["electroencephalography"],
    "ieeg": ["intracranial electroencephalography"],
    "physio": None, #TODO instance TBD
    "stim": None, #TODO instance TBD
    "beh": None, #TODO instance TBD
    "pet": ["positron emission tomography"],
    "2PE": ["two-photon fluorescence microscopy"],
    "BF": None, #TODO instance TBD
    "CARS": None, #TODO instance TBD
    "CONF": ["confocal microscopy"],
    "DIC": None, #TODO instance TBD
    "DF": None, #TODO instance TBD
    "FLUO": None, #TODO instance TBD
    "MPE": None, #TODO instance TBD
    "NLO": None, #TODO instance TBD
    "OCT": None, #TODO instance TBD
    "PC": None, #TODO instance TBD
    "PLI": ["polarized light microscopy"],
    "SEM": None, #TODO instance TBD
    "SPIM": None, #TODO instance TBD
    "SR": None, #TODO instance TBD
    "TEM": ["transmission electron microscopy"],
    "uCT": None, #TODO instance TBD
    "nirs": None, #TODO instance TBD
    "motion": None, #TODO instance TBD
}

MAP_2_UNITS = {
    "year": ["year"]
}

MAP_2_BIOLOGICALSEX = {
    "male": ["male"],
    "m": ["male"],
    "M": ["male"],
    "MALE": ["male"],
    "Male": ["male"],
    "female": ["female"],
    "f": ["female"],
    "F": ["female"],
    "FEMALE": ["female"],
    "Female": ["female"]
}

MAP_2_HANDEDNESS = {
    "left": ["left handedness"],
    "l": ["left handedness"],
    "L": ["left handedness"],
    "LEFT": ["left handedness"],
    "Left": ["left handedness"],
    "right": ["right handedness"],
    "r": ["right handedness"],
    "R": ["right handedness"],
    "RIGHT": ["right handedness"],
    "Right": ["right handedness"],
    "ambidextrous": ["ambidextrous handedness"],
    "a": ["ambidextrous handedness"],
    "A": ["ambidextrous handedness"],
    "AMBIDEXTROUS": ["ambidextrous handedness"],
    "Ambidextrous": ["ambidextrous handedness"]
}

MAP_2_SPECIES = {
    "homo sapiens": ["Homo sapiens"],
    "mus musculus": ["Mus musculus"],
    "rattus norvegicus": ["Rattus norvegicus"]
}


#sample_types = {
#    "cell line": None, #TODO instance TBD
#    "in vitro differentiated cells": None, #TODO instance TBD
#    "primary cell": None, #TODO instance TBD
#    "cell-free sample": None, #TODO instance TBD
#    "cloning host": None, #TODO instance TBD
#    "tissue": None, #TODO instance TBD
#    "whole organisms": None, #TODO instance TBD
#    "organoid": None, #TODO instance TBD
#    "technical sample": None #TODO instance TBD
#}



# test for create_openminds_person function in the main
import pytest
from bids2openminds.main import create_openminds_person
import openminds.v3.core as omcore

# Test data: (full_name, given_name, family_name)
example_names = [("John Ronald Reuel Tolkien", "John Ronald Reuel", "Tolkien"),
                 ("Bilbo Baggins", "Bilbo", "Baggins"),
                 ("Xue, G.", "G.", "Xue"),
                 ("Arndís Þórarinsdóttir", "Arndís", "Þórarinsdóttir"),
                 ("Loïc Le Clézio", "Loïc", "Le Clézio"),
                 ("P Gandolf", "P", "Gandolf")]

example_not_names = ["42", "#", "", "A34 hajb"]


@pytest.mark.parametrize("full_name, given_name, family_name", example_names)
def test_create_openminds_person(full_name, given_name, family_name):
    openminds_person_object = omcore.Person(given_name=given_name,
                                            family_name=family_name)
    bids2openminds_person_object = create_openminds_person(full_name)
    assert openminds_person_object.given_name == bids2openminds_person_object.given_name, \
        f"Given names don't match for input '{full_name}'"
    assert openminds_person_object.family_name == bids2openminds_person_object.family_name, \
        f"Family names don't match for input '{full_name}'"
    assert openminds_person_object.type_ == bids2openminds_person_object.type_, \
        f"Person types don't match for input '{full_name}'"
    # assert openminds_person_object == bids2openminds_person_object


@pytest.mark.parametrize("full_not_name", example_not_names)
def test_create_openminds_person_not_names(full_not_name):
    bids2openminds_person_object = create_openminds_person(full_not_name)
    assert bids2openminds_person_object is None
